@@ -61,11 +61,13 @@ StatusDisplayer *statusDisplayerGlobal;
 		// 15 is max (this is for libnjb devices)
 		[self setDebug:0];
 		[self setTurbo:YES];
+    [self setCreateAlbumFiles:YES];
 		// this defined here for the non MTP devices
 		batteryLevelMax = 100;
 		
 		downloadingTracks = NO;
 		downloadingFiles = NO;
+    cancelCurrentTransaction = NO;
 		
 		supportedFiletypes = NULL;
 		
@@ -244,7 +246,7 @@ StatusDisplayer *statusDisplayerGlobal;
 - (NJBTransactionResult *)connect
 {	
 	int n;
-	
+
 	connected = NO;
 	NSString *errorString = nil;
 	
@@ -252,7 +254,7 @@ StatusDisplayer *statusDisplayerGlobal;
 	
 	if (debug)
 		NJB_Set_Debug(debug);
-	
+  	
 	if (NJB_Discover(njbs, 0, &n) == -1)
 	{
 		//[self setStatus:STATUS_NO_NJB];
@@ -291,6 +293,8 @@ StatusDisplayer *statusDisplayerGlobal;
 	{
 		mtpDevice = YES;
 	  
+    //usb_set_debug(9);
+    
 		LIBMTP_Init();
 		device = LIBMTP_Get_First_Device();
     
@@ -372,7 +376,7 @@ StatusDisplayer *statusDisplayerGlobal;
 			NSLog(@"Could not get battery level property description");
       NSString *errorString = [self njbErrorString];
       NSLog(errorString);
-      return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Could not get battery level property description" extendedErrorString:errorString] autorelease];
+      //return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Could not get battery level property description" extendedErrorString:errorString] autorelease];
 		}
 		
 		LIBMTP_Get_Supported_Filetypes(device, &supportedFiletypes, &supportedFiletypesLength);
@@ -410,6 +414,8 @@ StatusDisplayer *statusDisplayerGlobal;
 		else
 		{
 			LIBMTP_Release_Device(device);
+      if (supportedFiletypes != NULL)
+        free(supportedFiletypes);
 		}
 		connected = NO;
 		[self setStatus:STATUS_DISCONNECTED];
@@ -590,7 +596,12 @@ StatusDisplayer *statusDisplayerGlobal;
 	{		
 		NSDate *date = [NSDate date];
 				
-		LIBMTP_track_t *tracks = LIBMTP_Get_Tracklisting_With_Callback(device, mtp_progress, &downloadingTracks);
+    ProgressParams progressParams;
+    progressParams.downloadingListing = &downloadingTracks;
+    BOOL bNo = NO;
+    progressParams.cancelCurrentTransaction = &bNo;
+    
+		LIBMTP_track_t *tracks = LIBMTP_Get_Tracklisting_With_Callback(device, mtp_progress, &progressParams);
 		if (tracks == NULL)
 		{
       NSString *errorString = [self njbErrorString];
@@ -630,6 +641,7 @@ StatusDisplayer *statusDisplayerGlobal;
 				NSCalendarDate *date = [[NSCalendarDate alloc] initWithString:nsTimeString calendarFormat:@"%Y%m%dT%H%M"];
 			
 				[myTrack setYear:[date yearOfCommonEra]];
+        [date release];
 			}
 			
 			[cachedTrackList addObject:myTrack];
@@ -723,9 +735,29 @@ StatusDisplayer *statusDisplayerGlobal;
 		if (trackmeta->filename != NULL)
 			free(trackmeta->filename);
 		trackmeta->filename = strdup(lastPathComponent);
-		
-		if (LIBMTP_Send_Track_From_File(device, filename, trackmeta, mtp_progress, NULL, [self musicAssoc]) != 0)
+      
+    // clear any errors we don't want so we can see if we get a cancellation error below
+    LIBMTP_Clear_Errorstack(device);
+      
+    ProgressParams progressParams;
+    BOOL bNo = NO;
+    progressParams.downloadingListing = &bNo;
+    cancelCurrentTransaction = NO;
+    progressParams.cancelCurrentTransaction = &cancelCurrentTransaction;
+    
+		if (LIBMTP_Send_Track_From_File(device, filename, trackmeta, mtp_progress, &progressParams, [self musicAssoc]) != 0)
 		{
+      LIBMTP_error_t *errorStack = LIBMTP_Get_Errorstack(device);
+      if (errorStack != NULL)
+      {
+        if (errorStack->errornumber == LIBMTP_ERROR_CANCELLED)
+        {
+          sleep(1);
+          
+          return [[[NJBTransactionResult alloc] initWithSuccess:YES
+                                                   resultString:NSLocalizedString(@"Cancelled", nil)] autorelease];	
+        }
+      }
 			NSString *error = @"Error uploading file";
 			NSLog(error);
       NSString *errorString = [self njbErrorString];
@@ -736,7 +768,8 @@ StatusDisplayer *statusDisplayerGlobal;
 		
 		[track setItemID:trackmeta->item_id];
     
-    [self addTrack:track ToAlbumWithName:[track album]];
+    if (createAlbumFiles)
+      [self addTrack:track ToAlbumWithName:[track album]];
 		
 		 LIBMTP_destroy_track_t(trackmeta);
 	}
@@ -857,8 +890,28 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		if (LIBMTP_Get_File_To_File(device, [track itemID], [[track fullPath] fileSystemRepresentation], mtp_progress, NULL) != 0)
+    ProgressParams progressParams;
+    BOOL bNo = NO;
+    progressParams.downloadingListing = &bNo;
+    cancelCurrentTransaction = NO;
+    progressParams.cancelCurrentTransaction = &cancelCurrentTransaction;
+    
+    // clear any errors we don't want so we can see if we get a cancellation error below
+    LIBMTP_Clear_Errorstack(device);
+    
+		if (LIBMTP_Get_File_To_File(device, [track itemID], [[track fullPath] fileSystemRepresentation], mtp_progress, &progressParams) != 0)
 		{
+      LIBMTP_error_t *errorStack = LIBMTP_Get_Errorstack(device);
+      if (errorStack != NULL)
+      {
+        if (errorStack->errornumber == LIBMTP_ERROR_CANCELLED)
+        {
+          sleep(1);
+          
+          return [[[NJBTransactionResult alloc] initWithSuccess:YES
+                                                   resultString:NSLocalizedString(@"Cancelled", nil)] autorelease];	
+        }
+      }
 			NSString *error = @"Error downloading file";
 			NSLog(error);
       NSString *errorString = [self njbErrorString];
@@ -1012,7 +1065,12 @@ StatusDisplayer *statusDisplayerGlobal;
 		
 		LIBMTP_file_t *files;
 		
-		files = LIBMTP_Get_Filelisting_With_Callback(device, mtp_progress, &downloadingFiles);
+    ProgressParams progressParams;
+    BOOL bNo = NO;
+    progressParams.downloadingListing = &downloadingFiles;
+    progressParams.cancelCurrentTransaction = &bNo;
+    
+		files = LIBMTP_Get_Filelisting_With_Callback(device, mtp_progress, &progressParams);
 		if (files == NULL)
 		{
 			downloadingFiles = NO;
@@ -1077,8 +1135,29 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		if (LIBMTP_Get_File_To_File(device, [dataFile itemID], [[dataFile fullPath] fileSystemRepresentation], mtp_progress, NULL) != 0)
+    ProgressParams progressParams;
+    BOOL bNo = NO;
+    progressParams.downloadingListing = &bNo;
+    cancelCurrentTransaction = NO;
+    progressParams.cancelCurrentTransaction = &cancelCurrentTransaction;
+    
+    // clear any errors we don't want so we can see if we get a cancellation error below
+    LIBMTP_Clear_Errorstack(device);
+    
+		if (LIBMTP_Get_File_To_File(device, [dataFile itemID], [[dataFile fullPath] fileSystemRepresentation], mtp_progress, &progressParams) != 0)
 		{
+      LIBMTP_error_t *errorStack = LIBMTP_Get_Errorstack(device);
+      if (errorStack != NULL)
+      {
+        if (errorStack->errornumber == LIBMTP_ERROR_CANCELLED)
+        {
+          sleep(1);
+          
+          return [[[NJBTransactionResult alloc] initWithSuccess:YES
+                                                   resultString:NSLocalizedString(@"Cancelled", nil)] autorelease];	
+        }
+      }
+      
 			NSString *error = @"Error downloading file";
 			NSLog(error);
       NSString *errorString = [self njbErrorString];
@@ -1167,8 +1246,29 @@ StatusDisplayer *statusDisplayerGlobal;
 		// todo: safe?
 		genfile->filename = strdup([[[dataFile fullPath] lastPathComponent] fileSystemRepresentation]);
 		
-		if (LIBMTP_Send_File_From_File(device, [[dataFile fullPath] fileSystemRepresentation], genfile, mtp_progress, NULL, parenthandle) != 0)
+    // clear any errors we don't want so we can see if we get a cancellation error below
+    LIBMTP_Clear_Errorstack(device);
+    
+    ProgressParams progressParams;
+    BOOL bNo = NO;
+    progressParams.downloadingListing = &bNo;
+    cancelCurrentTransaction = NO;
+    progressParams.cancelCurrentTransaction = &cancelCurrentTransaction;
+    
+		if (LIBMTP_Send_File_From_File(device, [[dataFile fullPath] fileSystemRepresentation], genfile, mtp_progress, &progressParams, parenthandle) != 0)
 		{
+      LIBMTP_error_t *errorStack = LIBMTP_Get_Errorstack(device);
+      if (errorStack != NULL)
+      {
+        if (errorStack->errornumber == LIBMTP_ERROR_CANCELLED)
+        {
+          sleep(1);
+          
+          return [[[NJBTransactionResult alloc] initWithSuccess:YES
+                                                   resultString:NSLocalizedString(@"Cancelled", nil)] autorelease];	
+        }
+      }
+      
 			/* we might get an (MTP error) object too large here */
 			NSString *error = @"Error uploading file";
 			NSLog(error);
@@ -1565,18 +1665,25 @@ int progress(u_int64_t sent, u_int64_t total, const char* buf, unsigned len, voi
 	return 0;
 }
 
-int mtp_progress (uint64_t const sent, uint64_t const total, void const * const data)
+int mtp_progress(uint64_t const sent, uint64_t const total, void const * const data)
 {
-  // if we're downloading a the file or track listing don't update on every file
-  if (data != NULL && *(BOOL*)data == YES)
+  ProgressParams *progressParams = (ProgressParams *)data;
+  // if we're downloading a the file or track listing don't update on every call
+  if (data != NULL && *(progressParams->downloadingListing))
   {
    if (sent % TRACKLIST_PROGRESS_UPDATE_INTERVAL != 0)
     return 0;
   }
 	double percent = (double)sent*100.0/(double)total;
 	[statusDisplayerGlobal updateTaskProgress:percent];
-	
-	return 0;
+
+  if (*(progressParams->cancelCurrentTransaction))
+  {
+    return 1;
+    *(progressParams->cancelCurrentTransaction) = NO;
+  }
+  else
+    return 0;
 }
 
 - (njb_songid_t *)songidStructFromTrack:(Track *)track
@@ -1819,7 +1926,7 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 	{
 		// they always charge...
 		// TODO: they don't - they can be fully charged. Can we see that?
-		return @"Charging";
+		return NSLocalizedString(@"Charging", nil);
 	}
 }
 
@@ -2035,6 +2142,8 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 		
   LIBMTP_album_t *libmtpAlbum = LIBMTP_new_album_t();
   libmtpAlbum->name = strdup([[album name] UTF8String]);
+  libmtpAlbum->artist = strdup([[track artist] UTF8String]);
+  libmtpAlbum->genre = strdup([[track genre] UTF8String]);
   libmtpAlbum->album_id = [album albumID];
   libmtpAlbum->no_tracks = size;
   libmtpAlbum->tracks = trackArray;
@@ -2069,50 +2178,6 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
         return nil;
       }
         
-      // let's set the artist too if we can
-      
-      uint32_t propnum = 0;
-      uint16_t *props = NULL;
-      uint32_t propcounter = 0;
-      int propfound = 0;
-      
-      uint16_t ret = ptp_mtp_getobjectpropssupported ((PTPParams *) device->params, PTP_OFC_MTP_AbstractAudioAlbum,
-                                                      &propnum, &props);
-      
-      if (ret != PTP_RC_OK || props == NULL) {
-        NSLog(@"could not get supported properties for type PTP_OFC_MTP_AbstractAudioAlbum.");
-        
-        NSString *errorString = [self njbErrorString];
-        NSLog(errorString);
-        
-        return nil;
-      }
-      
-      for (propcounter = 0; propcounter < propnum; propcounter++)
-      {
-        if (props[propcounter] == PTP_OPC_Artist)
-        {
-          propfound = 1;
-          break;
-        }
-      }
-      
-      free(props);
-      
-      if (propfound)
-      {
-        PTPPropertyValue val;
-        val.str = strdup([[track artist] UTF8String]);
-        ret = ptp_mtp_setobjectpropvalue ((PTPParams *) device->params, libmtpAlbum->album_id,
-                                          PTP_OPC_Artist, &val, PTP_DTC_STR);
-        
-        free(val.str);
-        
-        if (ret != PTP_RC_OK)
-          NSLog(@"could not set album artist, return %d", ret);
-        else
-          NSLog(@"set album artist name");
-      }
     }
   }
   else
@@ -2137,6 +2202,8 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 
 - (NJBTransactionResult *)uploadAlbumArt:(NSString *)filename forTrack:(Track *)track
 {
+  // be sure of thread safety
+  /*
   NSString *albumName = [track album];
   
   // don't add to a blank album
@@ -2190,8 +2257,19 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
     NSLog(errorString);
     return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error extendedErrorString:errorString] autorelease];
   }
-  
+  */
   return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
+- (void)setCreateAlbumFiles:(BOOL)newCreateAlbumFiles
+{
+  createAlbumFiles = newCreateAlbumFiles;
+}
+
+- (void)cancelCurrentTransaction
+{
+  cancelCurrentTransaction = YES;
+}
+
+
 @end
