@@ -6,13 +6,12 @@
 //
 
 #import <Cocoa/Cocoa.h>
-#import "MyMutableArray.h"
 #import "Track.h"
 #import "MyItem.h"
 
 @interface Playlist : MyItem {
 	unsigned playlistID;
-	MyMutableArray *tracks;
+	NSMutableArray *tracks;
 	NSString *name;
 	unsigned state;
 	NSLock *lock;
@@ -24,10 +23,10 @@
 - (void)addTrack:(Track *)track;
 - (void)deleteTrack:(Track *)track;
 - (void)deleteTracks:(NSArray *)tracksToDelete;
-- (MyMutableArray *)tracks;
+- (NSMutableArray *)tracks;
 - (NSString *)name;
 - (void)setName:(NSString *)newName;
-- (void)fillTrackInfoFromList:(MyMutableArray *)allTracks;
+- (void)fillTrackInfoFromList:(NSMutableArray *)allTracks;
 - (unsigned)trackCount;
 - (void)swapTrackWithPreviousAtIndex:(unsigned)index;
 - (void)swapTrackWithNextAtIndex:(unsigned)index;
