@@ -256,6 +256,18 @@
 	[defaults setObject:iTunesMusicLibraryLocation forKey:PREF_KEY_ITUNES_LOCATION];
 }
 
+- (BOOL)createAlbumFiles
+{
+  return [defaults boolForKey:PREF_KEY_CREATE_ALBUM_FILES];
+}
+
+- (void)setCreateAlbumFiles:(BOOL)createAlbumFiles
+{
+  [defaults setObject:[NSNumber numberWithBool:createAlbumFiles] forKey:PREF_KEY_CREATE_ALBUM_FILES]; 
+  
+  [theNJB setCreateAlbumFiles:createAlbumFiles];
+}
+
 /**********************/
 
 - (void)registerDefaults
@@ -304,6 +316,8 @@
 	[defaultValues setObject:DEFAULT_PREF_KEY_DOWNLOAD_DIR forKey:PREF_KEY_DOWNLOAD_DIR];
 	// ITUNES_LOCATION
 	[defaultValues setObject:DEFAULT_PREF_KEY_ITUNES_LOCATION forKey:PREF_KEY_ITUNES_LOCATION];
+  // CREATE_ALBUM_FILES
+  [defaultValues setObject:DEFAULT_PREF_KEY_CREATE_ALBUM_FILES forKey:PREF_KEY_CREATE_ALBUM_FILES];
 	
 	[defaults registerDefaults:defaultValues];
 }
