@@ -19,7 +19,7 @@ typedef enum _statusTypes {STATUS_CONNECTED = 0, STATUS_DISCONNECTED = 1, STATUS
 													 STATUS_DELETING_TRACK = 15, STATUS_DISCONNECTING = 16, STATUS_DELETING_FILE = 17,
 													 STATUS_DOWNLOADING_SETTINGS = 18, STATUS_UPDATING_SETTINGS = 19,
 													 STATUS_DOWNLOADING_PLAYLISTS = 20, STATUS_UPDATING_PLAYLIST = 21, STATUS_DELETING_PLAYLIST = 22,
-													 STATUS_CREATING_FOLDER = 23} statusTypes;
+													 STATUS_CREATING_FOLDER = 23, STATUS_UPLOADING_ALBUM_ART = 24} statusTypes;
 
 @interface StatusDisplayer : NSObject {
 	IBOutlet NSProgressIndicator *taskProgressIndicator;
