@@ -50,6 +50,8 @@
 	[album release];
 	[artist release];
 	[genre release];
+  [filename release];
+  [fullPath release];
 	[super dealloc];
 }
 
