@@ -791,7 +791,10 @@
 
 -(BOOL)setTitle:(NSString *)Title
 {
-	return [self setContent:[NSArray arrayWithObject:Title] forFrame:@"TT2" replace:YES];
+	if (Title == nil)
+		return [self setContent:nil forFrame:@"TT2" replace:YES];
+	else
+		return [self setContent:[NSArray arrayWithObject:Title] forFrame:@"TT2" replace:YES];
 /*
 	NSString *tempString;
 	switch (majorVersion)
