@@ -14,7 +14,7 @@
 #define __LIBNJB__H
 
 /** The version of this installation of libnjb */
-#define LIBNJB_VERSION 2.2.2
+#define LIBNJB_VERSION 2.2.4
 /** A legacy definition - nowadays we always compile for libusb */
 #define LIBNJB_COMPILED_FOR_LIBUSB 1
 
@@ -81,6 +81,8 @@ typedef unsigned __int64 u_int64_t;
 #define NJB_DEVICE_NJBZENMICRO	0x09 /**< Nomad Jukebox Zen Micro */
 #define NJB_DEVICE_DELLDJ2	0x0a /**< Second Generation Dell DJ */
 #define NJB_DEVICE_POCKETDJ     0x0b /**< Dell Pocket DJ */
+#define NJB_DEVICE_ZENSLEEK	0x0c /**< Zen Sleek */
+#define NJB_DEVICE_CREATIVEZEN  0x0d /**< Creative Zen (Micro variant) */
 /** @} */
 
 /**
@@ -144,7 +146,16 @@ typedef unsigned __int64 u_int64_t;
 #define NJB_UC_UTF8     1 /**< libnjb to use Unicode UTF-8 */
 /** @} */
 
-#define OWNER_STRING_LENGTH	128 /**< The fixed length of the owner string */
+/**
+ * @defgroup turboflags Turbo flags
+ * @see NJB_Set_Turbo_Mode()
+ * @{
+ */
+#define NJB_TURBO_OFF   0 /**< turbo mode is off for series 3 devices */
+#define NJB_TURBO_ON    1 /**< turbo mode is on for series 3 devices */
+
+/** The fixed length of the owner string */
+#define OWNER_STRING_LENGTH	128
 /** A type defined for owner strings */
 typedef unsigned char owner_string[OWNER_STRING_LENGTH + 1];
 
@@ -455,6 +466,7 @@ int NJB_Get_SDMI_ID(njb_t *njb, u_int8_t *sdmiid);
 const char *NJB_Get_Device_Name(njb_t *njb, int type);
 int NJB_Get_Firmware_Revision(njb_t *njb, u_int8_t *major, u_int8_t *minor, u_int8_t *release);
 int NJB_Get_Hardware_Revision(njb_t *njb, u_int8_t *major, u_int8_t *minor, u_int8_t *release);
+int NJB_Set_Turbo_Mode(njb_t *njb, u_int8_t mode);
 /**
  * @}
  * @defgroup tagapi The track and tag (song ID metadata) manipulation API
@@ -530,6 +542,7 @@ void NJB_Reset_Get_Datafile_Tag (njb_t *njb);
 njb_datafile_t *NJB_Get_Datafile_Tag (njb_t *njb);
 void NJB_Datafile_Destroy(njb_datafile_t *df);
 #define NJB_Get_File NJB_Get_Track
+#define NJB_Get_File_fd NJB_Get_Track_fd
 int NJB_Send_File (njb_t *njb, const char *path, const char *name, const char *folder,
 	NJB_Xfer_Callback *callback, void *data, u_int32_t *fileid);
 int NJB_Delete_Datafile (njb_t *njb, u_int32_t fileid);
